package cs636.music.presentation.web;

import java.util.Set;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import cs636.music.config.MusicSystemConfig;
import cs636.music.service.AdminService;
import cs636.music.service.ServiceException;
import cs636.music.service.data.DownloadData;
import cs636.music.service.data.InvoiceData;


@Controller
public class AdminController {

	@Autowired
	private AdminService adminService; 
	// All the admin-related URLs start with /adminController
	// This allows easy recognition of the special admin actions by pattern match on the URL
	// and allows filtering on the incoming request to make sure only admins can access 
	// these actions. See Chapter 16 of Murach and pg. 507 for a web.xml doing this kind
	// of security constraint (but on /admin).
	
	// Each group is related to an endpoint URL and its request handler
	// and the code following (request handlers) is in this order.
	
	private static final String ADMIN_BASE_URL = "/adminController/";
	private static final String ADMIN_JSP_DIR = "/WEB-INF/admin/";
	
	private static final String ADMIN_ERROR_VIEW = ADMIN_JSP_DIR + "error";
	
	private static final String ADMIN_WELCOME_URL = ADMIN_BASE_URL + "adminWelcome.html";
	private static final String ADMIN_MENU_VIEW = ADMIN_JSP_DIR + "adminMenu";
	private static final String ADMIN_LOGIN_VIEW = ADMIN_JSP_DIR + "adminLogin";
	
	private static final String ADMIN_LISTVARS_URL = ADMIN_BASE_URL + "listVariables.html";
	private static final String ADMIN_LISTVARS_VIEW = ADMIN_JSP_DIR + "listVariables";
	
	private static final String ADMIN_LOGOUT_URL = ADMIN_BASE_URL + "logout.html";
	private static final String ADMIN_LOGOUT_VIEW = ADMIN_JSP_DIR + "logout";
	
	private static final String ADMIN_INIT_DB_URL = ADMIN_BASE_URL + "initDB.html";
	private static final String ADMIN_INIT_DB_VIEW = ADMIN_JSP_DIR + "initDB";
	
	private static final String ADMIN_PROCESS_INVOICES_URL = ADMIN_BASE_URL + "processInvoices.html";
	private static final String ADMIN_PENDING_INVOICES_VIEW = ADMIN_JSP_DIR + "pendingInvoices";

	private static final String ADMIN_PROCESS_INVOICE_URL = ADMIN_BASE_URL + "processInvoice.html";
	// This handler doesn't have a view itself, it just forwards to ADMIN_PROCESS_INVOICES_URL
	
	private static final String ADMIN_DISPLAY_REPORTS_URL = ADMIN_BASE_URL + "displayReports.html";
	private static final String ADMIN_DISPLAY_REPORTS_VIEW = ADMIN_JSP_DIR + "displayReports";


	@RequestMapping(ADMIN_WELCOME_URL)
	public String handleAdminLogin(Model model, @RequestParam(required = false) String username,
			@RequestParam(required = false) String password, HttpServletRequest request) throws ServletException {
		String uname = (String) request.getSession().getAttribute("adminUser");
		if (uname != null) {
			return ADMIN_MENU_VIEW; // already logged in
		}

		if (username != null && password != null) {
			try {
				System.out.println("calling checkLogin...");
				Boolean adminExistence = adminService.checkLogin(username, password);
				// Success: Need to transfer the admin to the Admin Menu page
				// Bad credentials: Need to make the admin stay at the same login page.
				// Inform user 'Invalid Credentials'.
				if (adminExistence) {
					// save uname as session variable
					request.getSession().setAttribute("adminUser", username);
					return ADMIN_MENU_VIEW;
				} else {
					request.setAttribute("error", "Invalid Credentials");
					return ADMIN_LOGIN_VIEW;
				}
			} catch (ServiceException e) {
				System.out.println("error: "+e);
				String error = "Error in checking credentials: " + e;
				request.setAttribute("error", error);
				return ADMIN_ERROR_VIEW;
			}
		}
		// missing or incomplete username/password: show login page
		else {
			return ADMIN_LOGIN_VIEW;
		}
	}
	
	@RequestMapping(ADMIN_LISTVARS_URL)
	public String listVariables(Model model) {	
		String url = ADMIN_LISTVARS_VIEW;
		return url;
	}
	
	@RequestMapping(ADMIN_LOGOUT_URL)
	public String logout(Model model, HttpServletRequest request) {
		request.getSession().invalidate(); // drop session
		String url =ADMIN_LOGOUT_VIEW;
		return url;
	}
	
	@RequestMapping(ADMIN_INIT_DB_URL)
	public String initializeDB(Model model, HttpServletRequest request) throws ServletException {
		if (!checkAdmin(request))
			return "forward:" + ADMIN_WELCOME_URL;

		String info = null;
		try {
			adminService.initializeDB();
			info = "success";
		} catch (ServiceException e) {
			info = "failed: " + MusicSystemConfig.exceptionReport(e);
		}
		model.addAttribute("info", info);
		String url = ADMIN_INIT_DB_VIEW;
		return url;
	}

	@RequestMapping(ADMIN_BASE_URL + "processInvoices.html")
	public String processInvoices(Model model, HttpServletRequest request) throws ServletException {
		if (!checkAdmin(request))
			return "forward:" + ADMIN_WELCOME_URL;
		String url = null;
		try {
			Set<InvoiceData> pendingInvoices = adminService.getListofUnprocessedInvoices();
			model.addAttribute("unProcessedInvoices", pendingInvoices);
			System.out.println("#pendingInvoices: " + pendingInvoices.size());
			return ADMIN_PENDING_INVOICES_VIEW;
		} catch (ServiceException e) {
			String error = "Error: " + e;
			model.addAttribute("error", error);
			url = ADMIN_ERROR_VIEW;
		}
		return url;
	}
	
	@RequestMapping(ADMIN_PROCESS_INVOICE_URL)
	public String processInvoice(Model model, @RequestParam() long invoiceId, HttpServletRequest request)
			throws ServletException {
		if (!checkAdmin(request))
			return "forward:" + ADMIN_WELCOME_URL;
		String error = null;
		System.out.println(" Processing invoice " + invoiceId + " .....");
		try {
			adminService.processInvoice(invoiceId);
		} catch (ServiceException e) {
			error = "Error: " + e;
			model.addAttribute("error", error);
			return ADMIN_ERROR_VIEW;
		}
		return "forward:" + ADMIN_PROCESS_INVOICES_URL;
	}

	@RequestMapping(ADMIN_DISPLAY_REPORTS_URL)
	public String displayReports(Model model, HttpServletRequest request) throws ServletException {
		if (!checkAdmin(request))
			return "forward:" + ADMIN_WELCOME_URL;

		String url = null;
		String error = null;
		try {
			Set<DownloadData> downloadReport = adminService.getListofDownloads();
			model.addAttribute("downloads", downloadReport);
			Set<InvoiceData> invoiceReport = adminService.getListofInvoices();
			model.addAttribute("invoices", invoiceReport);
			url = ADMIN_DISPLAY_REPORTS_VIEW;
		} catch (ServiceException e) {
			error = "Error: " + e;
			model.addAttribute("error", error);
			url = ADMIN_ERROR_VIEW;
		}
		return url;
	}
	
	private boolean checkAdmin(HttpServletRequest request) {
		boolean isAdmin = (request.getSession().getAttribute("adminUser") != null);
		return isAdmin;
	}
}
